import { createRouter, createWebHistory } from "vue-router";
import Dashboard from "../views/Dashboard.vue";
import Login from "../views/Login.vue";
import Register from "../views/Register.vue";
import Article from "../views/Article.vue";
import DefaultLayout from "../components/DefaultLayout.vue";
import ArticleView from "../views/ArticleView.vue";
import AuthLayout from "../components/AuthLayout.vue";
import store from "../store/index.js"


const routes = [
  {
    path: '/',
    redirect: '/dashboard',
    component: DefaultLayout,
    meta: {reqiresAuth: true},
    children: [
      {path: '/dashboard', name: 'Dashboard', component: Dashboard},
      {path: '/article', name: "Article", component: Article},
      {path: '/article/create', name: "ArticleCreate", component: ArticleView},
      {path: '/article/:id', name: "ArticleView", component: ArticleView},
    ],

  },
  {
    path: "/auth",
    redirect: '/login',
    name: "Auth",
    component: AuthLayout,
    meta: {isGuest: true},
    children: [
      {
        path: "/login",
        name: "Login",
        component: Login
      },
      {
        path: "/register",
        name: "Register",
        component: Register
      },

    ]
  },

 ];

const router = createRouter({
  history: createWebHistory(),
  routes,
});

router.beforeEach((to, from, next) => {
  if (to.meta.requiresAuth && !store.state.user.token) {
    next({ name: "Login" });
  } else if (store.state.user.token && to.meta.isGuest) {
    next({ name: "Dashboard" });
  } else {
    next();
  }
});

export default router;